package cn.fw.morax.rpc.erp;

import cn.fw.data.base.domain.common.Message;
import cn.fw.ehr.sdk.api.ManagerApi;
import cn.fw.ehr.sdk.api.PostApi;
import cn.fw.ehr.sdk.api.StaffApi;
import cn.fw.ehr.sdk.api.result.*;
import cn.fw.erp.sdk.api.UserRoleApi;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.rpc.ehr.dto.PostInfoDTO;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleInfoDTO;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.cache.CacheKey;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : EhrRpcService
 * @description : 人事
 * @date: 2021-01-19 17:34
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class ErpRpcService {
    private final UserRoleApi userRoleApi;

    /**
     * 获取角色对应员工
     *
     * @param roleCode
     * @param shopId
     * @return
     */
    @Cacheable(cacheNames = "rpc_erp", key = "':rpc:erp:shop:roleCode:'+ #roleCode", unless = "#result == null")
    @Nullable
    public List<RpcUserRoleInfoDTO> getUserRoleInfo(String roleCode, Long shopId) {
        if (PublicUtil.isEmpty(roleCode)) {
            return new ArrayList<>();
        }
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleInfos(roleCode, shopId);
            log.info("获取角色编码和门店/商家对应员工信息 userRoleApi.queryUserRoleInfos 传入参数：roleCode:{}, value:{} 返回结果：{}", roleCode, shopId, JSON.toJSON(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return PublicUtil.copyList(msg.getData(), RpcUserRoleInfoDTO.class);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 获取角色对应员工
     *
     * @param userId
     * @param roleCode
     * @return
     */
    @Cacheable(cacheNames = "rpc_erp", key = "':rpc:erp:user:manager:code:'+ #roleCode", unless = "#result == null")
    @Nullable
    public List<Long> queryUserRoleShopIds(Long userId, String roleCode) {
        if (PublicUtil.isEmpty(roleCode)) {
            return new ArrayList<>();
        }
        try {
            Message<List<UserRoleDataRange>> msg = userRoleApi.queryUserRoleAllRange(userId, roleCode);
            log.info("查询数据范围内的角色用户信息 userRoleApi.queryUserRoleAllRange 传入参数：userId:{}, roleCode:{} 返回结果：{}", userId, roleCode, JSON.toJSON(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return msg.getData().stream()
                        .map(UserRoleDataRange::getRangeValue)
                        .collect(Collectors.toList());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 查询人员是否拥有某个角色
     *
     * @param userId
     * @param roleCode
     * @return
     */
    public boolean hasRole(Long userId, String roleCode) {
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleList(userId);
            log.info("获取人员角色信息 userRoleApi.queryUserRoleList 传入参数：userId:{} 返回结果：{}", userId, JSON.toJSONString(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return Optional.ofNullable(msg.getData())
                        .orElse(Lists.newArrayList())
                        .stream()
                        .anyMatch(userRole -> userRole.getRoleCode().equals(roleCode));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Boolean.FALSE;
    }

}
