package cn.fw.morax.server.controller.app;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.query.*;
import cn.fw.morax.domain.vo.SelectorVO;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.domain.vo.salary.SalaryGroupUserVO;
import cn.fw.morax.service.biz.kpi.KpiGroupBizService;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.biz.kpi.KpiPoolBizService;
import cn.fw.morax.service.biz.kpi.KpiStarSpecialRuleBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.annotation.IgnoreUserToken;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.time.YearMonth;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 绩效池查询
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/kpi")
@IgnoreAuth
public class KpiController {

    private final KpiStarSpecialRuleBizService kpiStarSpecialRuleBizService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final KpiPoolBizService kpiPoolBizService;
    private final KpiGroupBizService kpiGroupBizService;

    /**
     * 绩效池列表
     *
     * @param dto
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/staffs/pool")
    @ControllerMethod("绩效池列表")
    public Message<AppPage<KpiPoolVO>> queryKpiPool(@Validated KpiPoolQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getMonthly())) {
            dto.setMonthly(YearMonth.now());
        }
        return success(kpiGroupUserBizService.queryKpiPool(dto));
    }

    /**
     * 绩效池详情（查看他人绩效使用）
     *
     * @param kpiPoolId 绩效池id
     * @param yearMonth 月份
     * @return
     */
    @GetMapping("/pool/detail")
    @ControllerMethod("绩效池详情")
    public Message<KpiPoolDetailVO> queryKpiPoolDetail(@NotNull(message = "绩效池id不能为空") Long kpiPoolId,
                                                       @RequestParam(required = false) YearMonth yearMonth) {
        return success(kpiPoolBizService.queryPoolDetail(kpiPoolId, yearMonth));
    }

    /**
     * 绩效星级特殊规则详情
     *
     * @param KpiStarSpecialRuleId  绩效星级特殊规则id
     * @return
     */
    @GetMapping("/star-adjustment/detail")
    @ControllerMethod("绩效星级特殊规则详情")
    public Message<KpiStarSpecialRuleVO> getKpiStarSpecialRuleDetail(@NotNull(message = "绩效星级特殊规则id不能为空") Long KpiStarSpecialRuleId) {
        return success(kpiStarSpecialRuleBizService.getKpiStarSpecialRuleDetail(KpiStarSpecialRuleId));
    }

    /**
     * 绩效详情查询 (查看自己绩效详情)
     *
     * @param userId    用户id
     * @param yearMonth 月度
     * @return 绩效池详情
     * @ignoreParams userId
     */
    @GetMapping("/pool/own-detail")
    @ControllerMethod("绩效详情查询")
    public Message<KpiPoolDetailVO> kpiPoolOwnDetail(@CurrentUser Long userId,
                                                     @RequestParam(required = false) YearMonth yearMonth) {
        if (Objects.isNull(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        return success(kpiPoolBizService.queryUserPoolDetail(userId, yearMonth));
    }

    /**
     * 本月可查看的其他绩效池选项
     *
     * @param user      用户id
     * @param userId    用户id [管理层查看人员薪酬详情时需传值]
     * @param yearMonth 月度 [员工人查看自己的酬详情时需传值]
     * @return 薪酬池选项
     * @ignoreParams user
     */
    @GetMapping("/pool/selector")
    @IgnoreUserToken
    @ControllerMethod("本月可查看的其他绩效池选项")
    public Message<Set<SelectorVO>> salaryPoolSelector(@CurrentUser LoginAuthBean user, Long userId, @RequestParam(required = false) YearMonth yearMonth) {
        if (Objects.isNull(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        if (Objects.isNull(userId)) {
            userId = user.getUserId();
        }
        return success(kpiGroupUserBizService.queryUserKpiPool(userId, yearMonth));
    }

    /**
     * 员工绩效得分详情
     *
     * @param kpiPoolId 绩效池id
     * @param kpiGroupIndicatorId   绩效组指标id
     * @param yearMonth     月份
     * @return
     */
    @GetMapping("/kpi-indicator/score-detail")
    @IgnoreAuth
    @ControllerMethod("员工绩效得分详情")
    public Message<KpiGroupIndicatorDetailVO> getHitIndicatorLadder(Long kpiPoolId,
                                                                           Long kpiGroupIndicatorId,
                                                                           @RequestParam(required = false) YearMonth yearMonth) {
        if (PublicUtil.isEmpty(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        return success(kpiPoolBizService.getHitIndicatorLadder(kpiPoolId, kpiGroupIndicatorId, yearMonth));
    }


    /**
     * 查询绩效组不考核人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/kpi-ignored/staffs")
    @IgnoreAuth
    @ControllerMethod("查询绩效组不考核人员")
    public Message<AppPage<KpiGroupUserVO>> queryKpiIgnoredStaffs(@Validated KpiGroupUserPageDTO dto) {
        return success(kpiGroupUserBizService.queryKpiIgnoredStaffs(dto));
    }

    /**
     * 绩效池排名列表
     *
     * @param dto
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/staffs/pool-rank")
    @ControllerMethod("绩效池排名列表")
    public Message<AppPage<KpiPoolVO>> queryKpiPoolRank(@Validated KpiPoolRankQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getMonthly())) {
            dto.setMonthly(YearMonth.now());
        }
        return success(kpiGroupUserBizService.queryKpiPoolRank(dto));
    }

    /**
     * 查询绩效星级配置
     *
     * @param kpiGroupId
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/group/star-config")
    @ControllerMethod("查询绩效星级配置")
    public Message<List<KpiStarLaddersVO>> queryKpiGroup(Long kpiGroupId) {
        return success(kpiGroupUserBizService.queryKpiGroup(kpiGroupId));
    }

    /**
     * 绩效组详情
     *
     * @param id 绩效组id
     * @return
     */
    @GetMapping("/group-config")
    @ControllerMethod("绩效组详情")
    public Message<KpiGroupVO> kpiGroupDetail(@NotNull(message = "绩效组id不能为空") Long id) {
        return success(kpiGroupBizService.kpiGroupDetail(id));
    }

    /**
     * 实时查询绩效组人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/group-config/real-time/staffs")
    @ControllerMethod("实时查询绩效组人员")
    public Message<List<KpiGroupUserVO>> queryShopPostRealTimeStaffs(@Validated KpiGroupStaffQueryDTO dto) {
        return success(kpiGroupUserBizService.queryShopPostRealTimeStaffs(dto));
    }

}
