package cn.fw.morax.service.biz;

import cn.fw.common.exception.BusinessException;
import cn.fw.morax.common.utils.ExcelDataUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.AbstractLaddersDto;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.*;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@RequiredArgsConstructor
@Service
public class CommonService {

    private final EhrRpcService ehrRpcService;

    /**
     * 检查阶梯
     *
     * @param dtos
     */
    public static void checkLadders(List<? extends AbstractLaddersDto> dtos) {
        if (PublicUtil.isEmpty(dtos)) {
            return;
        }
        String errorPromptName = dtos.get(0).getBusinessName();
        checkLadders(dtos, errorPromptName);
    }

    /**
     * 检查阶梯
     *
     * @param dtos
     */
    public static void checkLadders(List<? extends AbstractLaddersDto> dtos, String errorPromptName) {
        BigDecimal lastUpper = null;
        //阶梯值校验  标准分不能超过指标基础分，阶梯值必须连续
        for (AbstractLaddersDto laddersDto : dtos) {
            BigDecimal lower = laddersDto.getLower();
            BigDecimal upper = laddersDto.getUpper();
            //上一条的下限要等于下一条的上限
            if (PublicUtil.isNotEmpty(lastUpper) && (lastUpper.compareTo(lower) != 0) ) {
                throw new BusinessException("【" + errorPromptName + "】的阶梯上限与下限不是连续，请编辑后重试");
            }
            //每条上限大于下限
            if (lower.compareTo(upper) >= 0) {
                throw new BusinessException("【" + errorPromptName + "】的阶梯下限大于或等于阶梯上限，请修改错误阶梯");
            }
            lastUpper = laddersDto.getUpper();
        }
    }


    /**
     * 导出文件
     *
     * @param response
     * @param rows
     * @param fileName
     */
    public static <T> void downloadExcel(HttpServletResponse response, List<T> rows, String fileName, Class<?> source) {
        ExcelWriter writer = ExcelUtil.getWriter();
        ExcelDataUtil.setExcelHeadAndWidth(writer, source, Boolean.TRUE);
        writer.setOnlyAlias(true);
        writer.write(rows, true);
        ExcelDataUtil.setAutoSizeColumn(writer.getSheet());
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.setHeader("Content-Disposition","attachment;filename=" + fileName);
        ServletOutputStream out= null;
        try {
            out = response.getOutputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        writer.flush(out, true);
        writer.close();
        IoUtil.close(out);
    }

    /**
     * 导出文件
     *
     * @param response
     * @param fileName
     */
    public static <T> void downloadTemplateExcel(HttpServletResponse response, String fileName, Class<?> source) {
        ExcelWriter writer = ExcelUtil.getWriter();
        Map<String, String> headMap = ExcelDataUtil.setExcelHeadAndWidth(writer, source, Boolean.FALSE);
        if (headMap.isEmpty()) {
            return;
        }
        writer.writeHeadRow(headMap.values());
        writer.setRowHeight(0, 25);
//        ExcelDataUtil.setAutoSizeColumn(writer.getSheet(), headMap.size());
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.setHeader("Content-Disposition","attachment;filename=" + fileName);
        ServletOutputStream out= null;
        try {
            out = response.getOutputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        writer.flush(out, true);
        writer.close();
        IoUtil.close(out);
    }

    /**
     * 获取编码后的文件名
     *
     * @param request
     * @param fileName
     * @return
     */
    public static String getEncodeName(HttpServletRequest request, String fileName) {
        String userAgent = request.getHeader("User-Agent");
        if (userAgent.contains("MSIE") || userAgent.contains("Trident") || userAgent.contains("Postman")) {
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        } else {// 非IE浏览器的处理：
            fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
        }
        return fileName;
    }

    /**
     * 实时查询在职员工
     *
     * @param userId
     * @return
     */
    public List<StaffBaseInfoDTO> queryRealTimeManageStaffs(Long userId) {
        List<StaffBaseInfoDTO> managerStaffs = ehrRpcService.queryManageStaffs(userId);
        if (PublicUtil.isEmpty(managerStaffs)) {
            managerStaffs = new ArrayList<>();
        }
        List<StaffBaseInfoDTO> selectVOS = managerStaffs.stream()
                .filter(staff -> (! staff.getId().equals(userId)))
                .collect(Collectors.toList());
        return selectVOS;
    }

    /**
     * 查询离职、在职员工
     *
     * @param userId
     * @param yearMonth
     * @return
     */
    public List<StaffBaseInfoDTO> queryManageStaffs(Long userId, YearMonth yearMonth) {
        List<StaffBaseInfoDTO> managerStaffs = ehrRpcService.queryManageStaffs(userId);
        if (PublicUtil.isEmpty(managerStaffs)) {
            managerStaffs = new ArrayList<>();
        }
        LocalDate startDate = LocalDate.of(yearMonth.getYear(), yearMonth.getMonth(), 1);
        LocalDate endDate = startDate.with(TemporalAdjusters.lastDayOfMonth());
        LocalDateTime startDateTime = LocalDateTime.of(startDate, LocalTime.MIN);
        LocalDateTime endDateTime = LocalDateTime.of(endDate, LocalTime.MAX);
        Date firstDayOfMonth = Date.from(startDateTime.atZone(ZoneId.systemDefault()).toInstant());
        Date endDayOfMonth = Date.from(endDateTime.atZone(ZoneId.systemDefault()).toInstant());
        List<StaffBaseInfoDTO> leaveManagerStaffs = ehrRpcService.queryLeaveManageStaffs(userId, firstDayOfMonth, endDayOfMonth);
        if (PublicUtil.isNotEmpty(leaveManagerStaffs)) {
            managerStaffs.addAll(leaveManagerStaffs);
        }
        List<StaffBaseInfoDTO> selectVOS = managerStaffs.stream()
                .filter(staff -> (! staff.getId().equals(userId)))
                .collect(Collectors.toList());
        return selectVOS;
    }

    /**
     * 查询离职、在职员工id
     *
     * @param userId
     * @param yearMonth
     * @return
     */
    public List<Long> queryManageStaffIds(Long userId, YearMonth yearMonth) {
        List<StaffBaseInfoDTO> staffBaseInfoDTOS = this.queryManageStaffs(userId, yearMonth);
        if (PublicUtil.isNotEmpty(staffBaseInfoDTOS)) {
            return staffBaseInfoDTOS.stream().map(StaffBaseInfoDTO::getId).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 查询离职员工id
     *
     * @param shopId
     * @param yearMonth
     * @return
     */
    public List<Long> queryShopLeaveStaffIds(Long shopId, YearMonth yearMonth) {
        List<Long> shopIds = new ArrayList<Long>(){{add(shopId);}};

        LocalDate startDate = LocalDate.of(yearMonth.getYear(), yearMonth.getMonth(), 1);
        LocalDate endDate = startDate.with(TemporalAdjusters.lastDayOfMonth());
        LocalDateTime startDateTime = LocalDateTime.of(startDate, LocalTime.MIN);
        LocalDateTime endDateTime = LocalDateTime.of(endDate, LocalTime.MAX);
        Date firstDayOfMonth = Date.from(startDateTime.atZone(ZoneId.systemDefault()).toInstant());
        Date endDayOfMonth = Date.from(endDateTime.atZone(ZoneId.systemDefault()).toInstant());

        List<StaffBaseInfoDTO> staffBaseInfoDTOS = ehrRpcService.queryShopLeaveStaff(shopIds, firstDayOfMonth, endDayOfMonth);
        if (PublicUtil.isNotEmpty(staffBaseInfoDTOS)) {
            return staffBaseInfoDTOS.stream().map(StaffBaseInfoDTO::getId).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

}
