package cn.fw.morax.service.biz.kpi;

import cn.fw.common.exception.BusinessException;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.kpi.KpiGroupCalculableBO;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import cn.fw.morax.service.biz.calculator.kpi.KpiAbsBaseCalculator;
import cn.fw.morax.service.data.kpi.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : KpiCalculateService
 * @description : 绩效计算类
 * @date : 2022-04-15 11:40
 */
@Slf4j
@Service
public class KpiCalculateService {
    private final KpiGroupService kpiGroupService;
    private final KpiPoolService kpiPoolService;
    private final KpiPoolIndicatorValueService kpiPoolIndicatorValueService;
    private final KpiGroupUserService kpiGroupUserService;
    private final KpiGroupIndicatorService kpiGroupIndicatorService;
    private final KpiRewardPointService kpiRewardPointService;
    private final StringRedisTemplate stringRedisTemplate;
    private final KpiPoolCommonService kpiPoolCommonService;
    private final Map<ScoreWayEnum, KpiAbsBaseCalculator> calculatorMap;

    @Value("${spring.cache.custom.global-prefix}:calculable:kpi-group")
    @Getter
    private String key;


    @Value("${spring.cache.custom.global-prefix}:calculate-scoring-rate:kpi-group")
    @Getter
    private String calcuScoringRateKey;

    @Autowired
    public KpiCalculateService(final KpiGroupService kpiGroupService,
                               final KpiPoolService kpiPoolService,
                               final KpiPoolIndicatorValueService kpiPoolIndicatorValueService,
                               final KpiGroupUserService kpiGroupUserService,
                               final KpiGroupIndicatorService kpiGroupIndicatorService,
                               final KpiRewardPointService kpiRewardPointService,
                               final StringRedisTemplate stringRedisTemplate,
                               final KpiPoolCommonService kpiPoolCommonService,
                               final List<KpiAbsBaseCalculator> kpiCalculatorList) {
        this.kpiGroupService = kpiGroupService;
        this.kpiPoolService = kpiPoolService;
        this.kpiPoolIndicatorValueService = kpiPoolIndicatorValueService;
        this.kpiGroupUserService = kpiGroupUserService;
        this.kpiGroupIndicatorService = kpiGroupIndicatorService;
        this.kpiRewardPointService = kpiRewardPointService;
        this.stringRedisTemplate = stringRedisTemplate;
        this.kpiPoolCommonService = kpiPoolCommonService;
        this.calculatorMap = kpiCalculatorList.stream().collect(Collectors.toMap(KpiAbsBaseCalculator::getCalMethod, v -> v));
    }

    /**
     * 缓存需要计算绩效的绩效组信息
     *
     * @param localDate 日期
     */
    public void cacheCalculableKpiGroupIds(final LocalDate localDate) {
        log.info("定时任务【缓存需要计算绩效的绩效组】开始执行");
        Set<Long> kpiGroupIds = kpiGroupService.queryCalculableKpiGroupIds(localDate);
        if (CollectionUtils.isEmpty(kpiGroupIds)) {
            return;
        }
        log.info("缓存需要计算绩效的绩效组信息，kpiGroupIds：{}，日期：{}", JSON.toJSONString(kpiGroupIds), localDate);
        cacheGroupIds(kpiGroupIds, localDate);
    }

    /**
     * 计算绩效
     */
    public void  calculateKpi() {
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(getKey());
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        List<String> overflowsList = new ArrayList<>();
        String str;
        while ((str = setOps.pop()) != null) {
            final KpiGroupCalculableBO bo = JSONObject.parseObject(str, KpiGroupCalculableBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                String finalStr = str;
                threadPool.execute(() -> {
                    try {
                        calculateKpi(bo);
                    } catch (Exception ex) {
                        log.error("计算绩效数据失败：{}", bo, ex);
                        setOps.add(finalStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsList.add(str);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsList)) {
            for (String s : overflowsList) {
                setOps.add(s);
            }
        }
    }

    /**
     * 计算对应日期绩效组的绩效数据
     * 按组计算
     *
     * @param bo
     */
    public void calculateKpi(KpiGroupCalculableBO bo) {
        final Long kpiGroupId = bo.getKpiGroupId();
        KpiGroup kpiGroup = kpiGroupService.getById(kpiGroupId);
        BV.notNull(kpiGroup, () -> "绩效组不存在");
        if (!Boolean.TRUE.equals(kpiGroup.getYn())) {
            log.error("无法计算被删除的绩效组数据: [{}]", kpiGroupId);
            return;
        }
        final LocalDate localDate = bo.getLocalDate();

        List<KpiGroupUser> userList = kpiGroupUserService.list(Wrappers.<KpiGroupUser>lambdaQuery()
                .eq(KpiGroupUser::getKpiGroupId, kpiGroupId)
                .eq(KpiGroupUser::getDataDate, localDate)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(userList)) {
            return;
        }
        for (KpiGroupUser kpiGroupUser : userList) {
            calcUserKpi(kpiGroupUser, kpiGroup);
        }
        BoundListOperations<String, String> listOps = stringRedisTemplate.boundListOps(getCalcuScoringRateKey());
        listOps.rightPush(JSONObject.toJSONString(bo));
    }

    /**
     * 计算绩效组人员的绩效数据
     * 计算单个人员绩效数据
     *
     * @param user
     */
    private void calcUserKpi(KpiGroupUser user, KpiGroup kpiGroup) {
        KpiPool kpiPool = kpiPoolCommonService.inspectionPool(user);

        if (Boolean.TRUE.equals(user.getIgnored())) {
            return;
        }
        final Long poolId = kpiPool.getId();
        List<KpiPoolIndicatorValue> indicatorValues = kpiPoolIndicatorValueService.list(Wrappers.<KpiPoolIndicatorValue>lambdaQuery()
                .eq(KpiPoolIndicatorValue::getUserId, user.getUserId())
                .eq(KpiPoolIndicatorValue::getDataDate, user.getDataDate())
                .eq(KpiPoolIndicatorValue::getKpiPoolId, poolId)
                .eq(KpiPoolIndicatorValue::getYn, Boolean.TRUE)
        );
//        boolean dataMissing = CollectionUtils.isEmpty(indicatorValues) || indicatorValues.size() >= kpiGroup.getIndicatorNum();
        boolean dataMissing = CollectionUtils.isEmpty(indicatorValues) || indicatorValues.size() > kpiGroup.getIndicatorNum();
        if (Boolean.TRUE.equals(dataMissing)) {
            log.error("业务指标数据不完整，绩效人员：{}，日期：{}，绩效池：{}，指标业务值条数：{}，绩效组：{}", user.getUserId(),
                    user.getDataDate(), poolId, indicatorValues.size(), JSON.toJSONString(kpiGroup));
            throw new BusinessException("业务指标数据不完整");
        }

        BigDecimal totalScore = BigDecimal.ZERO;
        for (KpiPoolIndicatorValue indicatorValue : indicatorValues) {
            KpiGroupIndicator groupIndicator = kpiGroupIndicatorService.getOne(Wrappers.<KpiGroupIndicator>lambdaQuery()
                            .eq(KpiGroupIndicator::getKpiGroupId, kpiGroup.getId())
                            .eq(KpiGroupIndicator::getIndicatorCode, indicatorValue.getIndicatorCode())
                            .eq(KpiGroupIndicator::getYn, Boolean.TRUE)
                    , Boolean.FALSE);
            if (PublicUtil.isEmpty(groupIndicator)) {
                log.error("绩效组指标:[{}]配置不存在", JSONObject.toJSONString(indicatorValue));
            }
            BV.notNull(groupIndicator, () -> "绩效组指标配置不存在");
            KpiAbsBaseCalculator calculator = calculatorMap.get(groupIndicator.getScoreWay());
            BV.notNull(calculator, () -> "绩效计算器不存在");
            BigDecimal result = calculator.calculate(indicatorValue, groupIndicator);
            indicatorValue.setIndicatorScore(result);
            totalScore = totalScore.add(result);
        }
        calcTotalScore(totalScore, kpiPool);
        kpiPoolIndicatorValueService.updateBatchById(indicatorValues);
        kpiPoolService.updateById(kpiPool);
    }




    private void cacheGroupIds(final Set<Long> kpiGroupIds, final LocalDate localDate) {
        String[] array = kpiGroupIds.stream()
                .map(kpiGroupId -> new KpiGroupCalculableBO(kpiGroupId, localDate))
                .map(JSONObject::toJSONString)
                .toArray(String[]::new);
        stringRedisTemplate.opsForSet().add(getKey(), array);
    }

    private void calcTotalScore(BigDecimal baseScore, KpiPool pool) {
        List<KpiRewardPoint> list = kpiRewardPointService.list(Wrappers.<KpiRewardPoint>lambdaQuery()
                .eq(KpiRewardPoint::getUserId, pool.getUserId())
                .eq(KpiRewardPoint::getPostId, pool.getPostId())
                .eq(KpiRewardPoint::getShopId, pool.getShopId())
                .eq(KpiRewardPoint::getMonthly, pool.getMonthly())
                .eq(KpiRewardPoint::getGroupId, pool.getGroupId())
        );
        if (CollectionUtils.isEmpty(list)) {
            pool.setKpiScore(baseScore);
            return;
        }

        BigDecimal resScore = list.stream().map(KpiRewardPoint::getScorePoint).reduce(BigDecimal.ZERO, BigDecimal::add);
        resScore = baseScore.add(resScore);
        if (resScore.compareTo(BigDecimal.ZERO) < 0) {
            resScore = BigDecimal.ZERO;
        }
        pool.setKpiScore(resScore);
    }
}
