package cn.fw.morax.service.biz.kpi;

import cn.fw.common.exception.BusinessException;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.kpi.KpiIndicatorBO;
import cn.fw.morax.domain.bo.kpi.UserIndicatorBO;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.enums.KpiTargetTypeEnum;
import cn.fw.morax.sdk.dto.CustomList;
import cn.fw.morax.sdk.dto.kpi.*;
import cn.fw.morax.service.component.KpiIndicatorReportProducer;
import cn.fw.morax.service.data.kpi.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : KpiReportService
 * @description : 绩效上报相关服务
 * @date : 2022-04-06 15:06
 */

@Slf4j
@RequiredArgsConstructor
@Service
public class KpiReportService {
    private final IndicatorsService indicatorsService;
    private final KpiGroupService kpiGroupService;
    private final KpiGroupIndicatorService kpiGroupIndicatorService;
    private final KpiGroupUserService kpiGroupUserService;
    private final KpiMqLogService kpiMqLogService;
    private final KpiIndicatorReportProducer kpiIndicatorReportProducer;
    private final KpiPoolIndicatorValueService kpiPoolIndicatorValueService;
    private final KpiPoolService kpiPoolService;
    private final KpiRewardPointService kpiRewardPointService;
    private final StringRedisTemplate stringRedisTemplate;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final KpiPoolCommonService kpiPoolCommonService;
    @Value("${spring.cache.custom.global-prefix}:indicator-value")
    @Getter
    private String indicatorKeyPref;

    /**
     * 通知业务系统上报数据
     * 有一天的延迟
     */
    public void sendNotice() {
        final Set<KpiIndicatorBO> set = kpiGroupIndicatorService.allIndicators(DateUtil.localDate2Date(LocalDate.now().minusDays(1L)));
        if (CollectionUtils.isEmpty(set)) {
            return;
        }
        for (KpiIndicatorBO bo : set) {
            try {
                sendNotice(bo, LocalDate.now().minusDays(1L));
            } catch (Exception e) {
                log.error("发送绩效上报mq失败: [{}]", JSON.toJSONString(bo), e);
            }
        }
    }

    /**
     * 手动 发送绩效数据上报mq
     *
     * @param indicatorCode
     * @param postId
     * @param groupId
     * @param localDate
     */
    public void sendNotice(final String indicatorCode, final Long postId, final String postName, final Long groupId, final LocalDate localDate) {
        List<KpiGroupIndicator> kpiGroupIndicators = kpiGroupIndicatorService.list(Wrappers.<KpiGroupIndicator>lambdaQuery()
                .eq(KpiGroupIndicator::getIndicatorCode, indicatorCode)
                .eq(KpiGroupIndicator::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(kpiGroupIndicators)) {
            throw new BusinessException("指标不存在");
        }
        KpiIndicatorBO bo = KpiIndicatorBO.builder()
                .indicatorCode(indicatorCode)
                .indicatorName(kpiGroupIndicators.get(0).getIndicatorName())
                .postId(postId)
                .postName(postName)
                .groupId(groupId)
                .build();
        sendNotice(bo, localDate);
    }

    /**
     * 发送绩效数据上报mq
     *
     * @param indicatorBO
     * @param localDate
     */
    @Transactional(rollbackFor = Exception.class)
    public void sendNotice(KpiIndicatorBO indicatorBO, final LocalDate localDate) {
        KpiMqLog mqlog = new KpiMqLog();
        mqlog.setIndicatorCode(indicatorBO.getIndicatorCode());
        mqlog.setPostId(indicatorBO.getPostId());
        mqlog.setSendDate(localDate);
        mqlog.setGroupId(indicatorBO.getGroupId());
        kpiMqLogService.save(mqlog);

        kpiIndicatorReportProducer.send(KpiReportNoticeMQ.builder()
                .indicatorCode(indicatorBO.getIndicatorCode())
                .indicatorName(indicatorBO.getIndicatorName())
                .postId(indicatorBO.getPostId())
                .postName(indicatorBO.getPostName())
                .groupId(indicatorBO.getGroupId())
                .uploadDate(DateUtil.localDate2Date(localDate))
                .uid(mqlog.getId())
                .build());

    }

    @Transactional(rollbackFor = Exception.class)
    public void reportKpiReward(KpiPointRewardReq req) {
        LocalDateTime occurredTime = DateUtil.date2LocalDateTime(req.getDataTime());
        KpiRewardPoint kpiRewardPoint = new KpiRewardPoint();
        BeanUtils.copyProperties(req, kpiRewardPoint);
        kpiRewardPoint.setOccurredTime(occurredTime);
        kpiRewardPoint.setMonthly(YearMonth.from(occurredTime));
        if (!Boolean.TRUE.equals(req.getAward())) {
            kpiRewardPoint.setScorePoint(req.getScorePoint().multiply(BigDecimal.valueOf(-1)));
        }
        kpiRewardPointService.save(kpiRewardPoint);
    }

    /**
     * 收集业务系统上报的数据
     * 先直接放到redis增加系统吞吐量
     *
     * @param userIndicatorList
     */
    public void reportKpiData(CustomList<UserIndicatorReq> userIndicatorList) {
        String[] jsonArr = userIndicatorList.stream().map(r -> {
            UserIndicatorBO bo = new UserIndicatorBO();
            BeanUtils.copyProperties(r, bo);
            return JSONObject.toJSONString(bo);
        }).toArray(String[]::new);
        BoundListOperations<String, String> opsList = stringRedisTemplate.boundListOps(getIndicatorKeyPref());
        opsList.rightPushAll(jsonArr);
    }

    /**
     * 从缓存取池上报的数据并进行持久化
     */
    public void persistenceUserKpiData() {
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        ListOperations<String, String> opsForList = stringRedisTemplate.opsForList();
        List<String> overflowsList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = opsForList.leftPop(getIndicatorKeyPref())) != null) {
            final UserIndicatorBO bo = JSONObject.parseObject(jsonStr, UserIndicatorBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                //保存业务系统绩效指标值
                String finalJsonStr = jsonStr;
                threadPool.execute(() -> {
                    try {
                        saveUserIndicator(bo);
                    } catch (Exception e) {
                        log.error("保存业务系统绩效指标值失败：{}", bo, e);
                        opsForList.rightPushAll(getIndicatorKeyPref(), finalJsonStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsList.add(jsonStr);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsList)) {
            opsForList.rightPushAll(getIndicatorKeyPref(), overflowsList);
        }
    }


    /**
     * 根据岗位码和绩效指标码查询对应日期的绩效组人员
     *
     * @param kpiGroupUserQuery
     * @return 绩效组人员
     */
    public List<KpiGroupUserResult> getIndicatorUser(final KpiGroupUserQuery kpiGroupUserQuery) {
        final String indicatorCode = kpiGroupUserQuery.getIndicatorCode();
        final Long postId = kpiGroupUserQuery.getPostId();
        final Date dataDate = new Date(kpiGroupUserQuery.getDataDate());
        log.info("查询绩效组人员，日期：{}", new SimpleDateFormat("yyyy-MM-dd").format(dataDate));
        Indicators indicator = indicatorsService.queryByCode(indicatorCode, Boolean.FALSE);
        BV.notNull(indicator, () -> "指标码不存在或已失效");

        ArrayList<KpiGroupUserResult> list = new ArrayList<>();

        List<Long> kpiGroupIdArr = kpiGroupService.queryKgiGroupIdByIndCode(indicatorCode,
                postId, kpiGroupUserQuery.getGroupId(), dataDate);

        if (CollectionUtils.isEmpty(kpiGroupIdArr)) {
            log.error("没有对应的绩效组数据【{}】", kpiGroupUserQuery);
            return list;
        }

        List<KpiGroupUser> userList = kpiGroupUserService.list(Wrappers.<KpiGroupUser>lambdaQuery()
                .in(KpiGroupUser::getKpiGroupId, new HashSet<>(kpiGroupIdArr))
                .eq(KpiGroupUser::getDataDate, DateUtil.date2LocalDate(dataDate))
                .ne(KpiGroupUser::getIgnored, Boolean.TRUE)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );

        if (CollectionUtils.isEmpty(userList)) {
            log.error("没有对应日期的绩效组人员数据【{}】", DateUtil.getStringDateShort(dataDate));
            return list;
        }

        Map<Long, List<KpiGroupUser>> kpiGroupUserMap = userList.stream().collect(Collectors.groupingBy(KpiGroupUser::getKpiGroupId));
        for (Map.Entry<Long, List<KpiGroupUser>> entry : kpiGroupUserMap.entrySet()) {
            KpiGroupIndicator groupIndicator = kpiGroupIndicatorService.getOne(Wrappers.<KpiGroupIndicator>lambdaQuery()
                            .eq(KpiGroupIndicator::getKpiGroupId, entry.getKey())
                            .eq(KpiGroupIndicator::getIndicatorCode, indicatorCode)
                            .eq(KpiGroupIndicator::getYn, Boolean.TRUE)
                    , Boolean.FALSE);

            KpiGroupUserResult result = new KpiGroupUserResult();
            result.setRoleCodeList(indicator.getRoleCodes());
            result.setUserIdList(transferUser(entry.getValue()));
            result.setTargetType(TargetTypeEnum.ofValue(groupIndicator.getTargetType().getValue()));
            if (! KpiTargetTypeEnum.NO.equals(groupIndicator.getTargetType())) {
                result.setTargetValue(groupIndicator.getTargetValue());
            }
            list.add(result);
        }

        return list;
    }


    /**
     * 根据岗位码和绩效指标码查询对应日期的绩效组人员
     *
     * @param kpiGroupUserQuery
     * @return 绩效组人员
     */
    public List<KpiGroupUserResult> getRealTimeIndicatorUser(final KpiGroupUserQuery kpiGroupUserQuery) {
        final String indicatorCode = kpiGroupUserQuery.getIndicatorCode();
        final Long postId = kpiGroupUserQuery.getPostId();
        final Date dataDate = new Date();
        final LocalDate localDate = DateUtil.date2LocalDate(dataDate);
        Indicators indicator = indicatorsService.queryByCode(indicatorCode, Boolean.FALSE);
        BV.notNull(indicator, () -> "指标码不存在或已失效");

        ArrayList<KpiGroupUserResult> list = new ArrayList<>();

        List<KpiGroup> kpiGroups = kpiGroupService.queryKgiGroupByIndCode(indicatorCode,
                postId, kpiGroupUserQuery.getGroupId(), dataDate);

        if (CollectionUtils.isEmpty(kpiGroups)) {
            log.error("没有对应的绩效组数据【{}】", kpiGroupUserQuery);
            return list;
        }
        List<Long> kpiGroupIds = kpiGroups.stream().map(KpiGroup::getId).collect(Collectors.toList());
        List<KpiGroupUser> groupUsers = kpiGroupUserService.list(Wrappers.<KpiGroupUser>lambdaQuery()
                .in(KpiGroupUser::getKpiGroupId, new HashSet<>(kpiGroupIds))
                .eq(KpiGroupUser::getDataDate, localDate)
                .ne(KpiGroupUser::getIgnored, Boolean.TRUE)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );

        this.mergeRealTimeUser(postId, kpiGroups, groupUsers, localDate);

        if (CollectionUtils.isEmpty(groupUsers)) {
            log.error("没有对应日期的绩效组人员数据【{}】", DateUtil.getStringDateShort(dataDate));
            return list;
        }

        Map<Long, List<KpiGroupUser>> kpiGroupUserMap = groupUsers.stream().collect(Collectors.groupingBy(KpiGroupUser::getKpiGroupId));
        for (List<KpiGroupUser> valueArr : kpiGroupUserMap.values()) {
            KpiGroupUserResult result = new KpiGroupUserResult();
            result.setRoleCodeList(indicator.getRoleCodes());
            result.setUserIdList(transferUser(valueArr));
            list.add(result);
        }

        return list;
    }

    /**
     * 合并绩效组实时员工
     *
     * @param postId
     * @param kpiGroups
     * @param userList
     * @return
     */
    public void mergeRealTimeUser(Long postId, List<KpiGroup> kpiGroups, List<KpiGroupUser> userList, LocalDate localDate) {
        List<KpiGroupUser> groupUsers = kpiGroupUserBizService.fetchBuildKpiUser(postId, kpiGroups, localDate);
        groupUsers = groupUsers.stream().filter(user -> Boolean.FALSE.equals(user.getIgnored())).collect(Collectors.toList());
        if (PublicUtil.isEmpty(userList)) {
            userList.addAll(groupUsers);
            return;
        }
        Set<Long> userIds = userList.stream().map(KpiGroupUser::getUserId).collect(Collectors.toSet());
        List<KpiGroupUser> notExistUser = groupUsers.stream().filter(user -> (! userIds.contains(user.getUserId()))).collect(Collectors.toList());
        userList.addAll(notExistUser);
    }

    /**
     * 持久化业务系统上报的指标数据
     *
     * @param bo
     * @throws cn.fw.common.exception.BusinessException 校验数据异常
     */
    @Transactional(rollbackFor = Exception.class)
    protected void saveUserIndicator(UserIndicatorBO bo) {
        LocalDate dataDate = DateUtil.date2LocalDate(bo.getDataDate());
        //先通过绩效组获取绩效池，绩效池不存在就创建，绩效池存在检查绩效池在职门店与员工在职门店是否一致
        KpiGroup kpiGroup = kpiGroupService.getEffectGroup(bo.getShopId(), bo.getPostId(), bo.getDataDate());
        if (PublicUtil.isEmpty(kpiGroup)) {
            log.error("上报门店、岗位不存在绩效组，上报数据：{}", JSON.toJSONString(bo));
            throw new BusinessException("绩效数据上报，绩效组不存在");
        }

        KpiGroupUser groupUser = kpiGroupUserService.getOne(Wrappers.<KpiGroupUser>lambdaQuery()
                .eq(KpiGroupUser::getShopId, bo.getShopId())
                .eq(KpiGroupUser::getPostId, bo.getPostId())
                .eq(KpiGroupUser::getUserId, bo.getUserId())
                .eq(KpiGroupUser::getDataDate, dataDate)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );

        KpiPool kpiPool = kpiPoolCommonService.inspectionPool(groupUser);
        BV.notNull(kpiPool, () -> "用户绩效池不存在");

        if (PublicUtil.isEmpty(groupUser)) {
            log.error("上报门店、岗位不存在绩效人员，上报数据：{}", JSON.toJSONString(bo));
            throw new BusinessException("绩效数据上报，绩效人员不存在");
        }

        kpiPoolIndicatorValueService.update(Wrappers.<KpiPoolIndicatorValue>lambdaUpdate()
                .set(KpiPoolIndicatorValue::getYn, Boolean.FALSE)
                .eq(KpiPoolIndicatorValue::getUserId, bo.getUserId())
                .eq(KpiPoolIndicatorValue::getDataDate, DateUtil.date2LocalDate(bo.getDataDate()))
                .eq(KpiPoolIndicatorValue::getIndicatorCode, bo.getIndicatorCode())
        );
        KpiPoolIndicatorValue kpiValue = new KpiPoolIndicatorValue();
        kpiValue.setKpiPoolId(kpiPool.getId());
        kpiValue.setUserId(bo.getUserId());
        kpiValue.setIndicatorCode(bo.getIndicatorCode());
        kpiValue.setIndicatorValue(bo.getValue());
        kpiValue.setDataDate(dataDate);
        kpiValue.setMqLogId(bo.getUid());
        kpiValue.setGroupId(kpiPool.getGroupId());
        kpiValue.setYn(Boolean.TRUE);
        kpiValue.setIndicatorScore(BigDecimal.ZERO);
        kpiPoolIndicatorValueService.save(kpiValue);
    }

    /**
     * 转换器  KpiGroupUser -> SimpleUserData
     *
     * @param userList
     * @return
     */
    private List<KpiGroupUserResult.SimpleUserData> transferUser(List<KpiGroupUser> userList) {
        List<KpiGroupUserResult.SimpleUserData> list = new ArrayList<>();
        for (KpiGroupUser kpiGroupUser : userList) {
            KpiGroupUserResult.SimpleUserData simpleUserData = new KpiGroupUserResult.SimpleUserData();
            simpleUserData.setUserId(kpiGroupUser.getUserId());
            simpleUserData.setUserName(kpiGroupUser.getUserName());
            simpleUserData.setShopId(kpiGroupUser.getShopId());
            simpleUserData.setShopName(kpiGroupUser.getShopName());
            list.add(simpleUserData);
        }

        return list;
    }
}
