package cn.fw.morax.service.biz.salary;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.morax.common.pojo.event.PaySalaryEvent;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.EventBusUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.SalaryConfirm;
import cn.fw.morax.domain.db.salary.SalaryGeneralSettin;
import cn.fw.morax.domain.db.salary.SalaryPool;
import cn.fw.morax.domain.enums.SalaryConfirmStatusEnum;
import cn.fw.morax.rpc.backlog.TodoRpcService;
import cn.fw.morax.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.morax.service.data.salary.SalaryConfirmService;
import cn.fw.morax.service.data.salary.SalaryGeneralSettinService;
import cn.fw.morax.service.data.salary.SalaryPoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.locks.Lock;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryCalcService
 * @description : 薪资确认类
 * @date : 2022-04-25 16:24
 */

@Slf4j
@Service
@RequiredArgsConstructor
public class SalaryConfirmBizService {
    private final SalaryGeneralSettinService salaryGeneralSettinService;
    private final SalaryConfirmService salaryConfirmService;
    private final SalaryPoolService salaryPoolService;
    private final DistributedLocker distributedLocker;
    private final TodoRpcService todoRpcService;


    @Value("${spring.cache.custom.global-prefix}:salary:confirm:")
    @Getter
    private String keyPrefix;

    @Value("${todo.person-confirm}")
    @Getter
    private String todoCode;
    @Value("${todo.pay-salary-notice}")
    @Getter
    private String paySalaryTodoCode;

    /**
     * 确认薪资
     *
     * @param id
     * @param fid
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "#id", message = "确认中，请勿重复操作")
    public void confirmSalary(Long id, String fid) {
        SalaryConfirm confirm = salaryConfirmService.getById(id);
        BV.notNull(confirm, () -> "数据异常，请刷新重试");
        boolean flag = SalaryConfirmStatusEnum.NEED_CONFIRM.equals(confirm.getConfirmedStatus()) && Objects.isNull(confirm.getConfirmTime());
        BV.isTrue(flag, () -> "该数据以及确认，请勿重复确认");
        SalaryPool pool = salaryPoolService.getById(confirm.getSalaryPoolId());
        BV.notNull(pool, () -> "薪酬数据异常，请刷新重试");
        confirm.setConfirmTime(LocalDateTime.now());
        confirm.setConfirmedStatus(SalaryConfirmStatusEnum.CONFIRMED);
        confirm.setSelfConfirm(Boolean.TRUE);
        pool.setSignatureFile(fid);
        BackLogItemDTO dto = new BackLogItemDTO(pool.getUserId(), confirm.getTodoCode(), String.valueOf(confirm.getId()), new Date(), pool.getShopId());
        boolean complete = todoRpcService.complete(dto);
        BV.isTrue(complete, () -> "确认失败，请稍后重试");
        salaryConfirmService.updateById(confirm);
        salaryPoolService.updateById(pool);

        EventBusUtil.asyncPost(PaySalaryEvent.builder()
                .monthly(pool.getMonthly())
                .groupId(pool.getGroupId())
                .shopId(pool.getShopId())
                .shopName(pool.getShopName())
                .build());
    }


    /**
     * 生成薪资确认待办（申述日期开始时发送）
     *
     * @param groupId
     */
    public void createSalaryConfirmTodo(Long groupId) {
        String key = String.format("%s:%s", ":create-salary-confirm-todo", groupId);
        Lock lock = distributedLocker.lock(key);
        if (!((RLock) lock).isLocked()) {
            return;
        }
        try {
            SalaryGeneralSettin settin = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaQuery()
                            .eq(SalaryGeneralSettin::getYn, Boolean.TRUE)
                            .eq(SalaryGeneralSettin::getGroupId, groupId)
                    , Boolean.FALSE);
            if (Objects.isNull(settin)) {
                settin = salaryGeneralSettinService.initData(groupId);
            }
            String datesOfAppeal = Optional.ofNullable(settin.getDatesOfAppeal()).orElse("1,5");
            String[] daysArr = datesOfAppeal.split(",");
            int day = Integer.parseInt(daysArr[1]);
            final LocalDateTime planTime = YearMonth.now().atDay(day).atTime(9, 0);
            int payoffDate = Optional.ofNullable(settin.getPayoffDate()).orElse(5);
            final LocalDateTime deadline = YearMonth.now().atDay(payoffDate).atStartOfDay().minusSeconds(1L);
            createConfirms(planTime, deadline, groupId);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 推送待办
     */
    public void pushConfirm() {
        List<SalaryConfirm> list = salaryConfirmService.list(Wrappers.<SalaryConfirm>lambdaQuery()
                .ge(SalaryConfirm::getPlanTime, LocalDateTime.now().minusDays(3L))
                .le(SalaryConfirm::getDeadline, LocalDate.now().plusDays(1L).atStartOfDay())
                .eq(SalaryConfirm::getPushedTodo, Boolean.FALSE)
                .eq(SalaryConfirm::getYn, Boolean.TRUE)
                .last(" limit 500 ")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (SalaryConfirm salaryConfirm : list) {
            executeSendTodo(salaryConfirm);
        }
        salaryConfirmService.updateBatchById(list);
    }

    private void createConfirms(final LocalDateTime planTime, final LocalDateTime deadline, final Long groupId) {
        List<SalaryPool> list = salaryPoolService.list(Wrappers.<SalaryPool>lambdaQuery()
                .eq(SalaryPool::getMonthly, YearMonth.now().minusMonths(1L))
                .eq(SalaryPool::getGroupId, groupId)
                .eq(SalaryPool::getPaid, Boolean.FALSE)
                .eq(SalaryPool::getRegular, Boolean.TRUE)
                .eq(SalaryPool::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<SalaryConfirm> confirmList = new ArrayList<>();
        for (SalaryPool pool : list) {
            int count = salaryConfirmService.count(Wrappers.<SalaryConfirm>lambdaQuery()
                    .eq(SalaryConfirm::getSalaryPoolId, pool.getId())
                    .eq(SalaryConfirm::getYn, Boolean.TRUE)
            );
            if (count > 0) {
                continue;
            }
            SalaryConfirm confirm = new SalaryConfirm();
            confirm.setSalaryPoolId(pool.getId());
            confirm.setPlanTime(planTime);
            confirm.setPushedTodo(Boolean.FALSE);
            confirm.setDeadline(deadline);
            confirm.setConfirmedStatus(SalaryConfirmStatusEnum.NEED_CONFIRM);
            confirm.setSelfConfirm(Boolean.FALSE);
            confirm.setGroupId(groupId);
            confirm.setYn(Boolean.TRUE);
            confirm.setTodoCode(getTodoCode());
            confirmList.add(confirm);
        }
        if (PublicUtil.isNotEmpty(confirmList)) {
            salaryConfirmService.saveBatch(confirmList);
        }
    }

    private void executeSendTodo(SalaryConfirm confirm) {
        SalaryPool salaryPool = salaryPoolService.getById(confirm.getSalaryPoolId());
        if (Objects.isNull(salaryPool)) {
            return;
        }
        try {
            Map<String, String> dynamicMap = new HashMap<>(5);
            dynamicMap.put("userName", salaryPool.getUserName());
            dynamicMap.put("postName", salaryPool.getPostName());
            dynamicMap.put("shopName", salaryPool.getShopName());
            dynamicMap.put("monthly", salaryPool.getMonthly().toString());
            dynamicMap.put("poolId", String.valueOf(salaryPool.getId()));
            dynamicMap.put("salaryConfirmId", String.valueOf(confirm.getId()));
            BackLogItemDTO dto = new BackLogItemDTO(salaryPool.getUserId(), confirm.getTodoCode(), String.valueOf(confirm.getId()), DateUtil.localDateTime2Date(confirm.getPlanTime()), salaryPool.getShopId());
            dto.setExpireTime(DateUtil.localDateTime2Date(confirm.getDeadline()));
            dto.setDynamicMap(dynamicMap);
            boolean pushed = todoRpcService.push(dto);
            if (pushed) {
                confirm.setPushedTodo(Boolean.TRUE);
            }
        } catch (Exception e) {
            log.error("推送待办失败，dataId:{}", confirm.getId(), e);
        }
    }

}
