package cn.fw.morax.service.biz.salary;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.db.kpi.KpiGroupUser;
import cn.fw.morax.domain.db.kpi.KpiPool;
import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.dto.SalaryGroupDTO;
import cn.fw.morax.domain.dto.SalaryGroupProjectDTO;
import cn.fw.morax.domain.dto.SalaryGroupProjectSettinDTO;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.service.data.salary.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.YearMonth;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class SalaryGroupDataService {

    private final SalaryGroupProjectSettinService salaryGroupProjectSettinService;
    private final SalaryGroupProjectService salaryGroupProjectService;
    private final SalaryGroupService salaryGroupService;
    private final SalaryGroupUserService salaryGroupUserService;
    private final SalaryPoolService salaryPoolService;

    /**
     * 保存薪酬组
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public SalaryGroup saveSalaryGroup(SalaryGroupDTO dto) {
        SalaryGroup salaryGroup = PublicUtil.copy(dto, SalaryGroup.class);
        salaryGroup.setProjectNum(dto.getProjects().size());
        if (PublicUtil.isNotEmpty(dto.getId())) {
            salaryGroup.setPreId(dto.getId());
            salaryGroup.setSgc(dto.getSgc());
        } else {
            salaryGroup.setSgc(PublicUtil.getUUID());
        }
        salaryGroup.setStatus(SettingStatusEnum.APPROVING);
        salaryGroupService.save(salaryGroup);
        return salaryGroup;
    }

    /**
     * 保存薪酬指标、阶梯
     *
     * @param projects
     * @param salaryGroup
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveSalaryGroupProjects(List<SalaryGroupProjectDTO> projects, SalaryGroup salaryGroup) {
        List<SalaryGroupProjectSettin> salaryGroupProjectSettings = Lists.newArrayListWithCapacity(30);
        Long projectId = null;
        SalaryGroupProject project = null;
        SalaryGroupProjectSettin setting = null;
        for (SalaryGroupProjectDTO projectDTO : projects) {
            project = PublicUtil.copy(projectDTO, SalaryGroupProject.class);
            project.setSalaryGroupId(salaryGroup.getId());
            salaryGroupProjectService.save(project);
            projectId = project.getId();
            Boolean isLadderSetting = Boolean.FALSE;
            if (SalaryCalMethodEnum.LADDER_FIXED.equals(projectDTO.getCalMethod()) ||
                    SalaryCalMethodEnum.LADDER_SINGLE.equals(projectDTO.getCalMethod())) {
                isLadderSetting = Boolean.TRUE;
            }
            for (SalaryGroupProjectSettinDTO settingDTO : projectDTO.getSettings()){
                setting = PublicUtil.copy(settingDTO, SalaryGroupProjectSettin.class);
                setting.setSalaryGroupProjectId(projectId);
                setting.setStairKey(settingDTO.getStairKey());
                setting.setStairKeyDesc(settingDTO.getStairKeyDesc());
                setting.setStairValue(settingDTO.getStairValue());
                if (isLadderSetting) {
                    setting.setStairMin(settingDTO.getStairMin().intValue());
                    setting.setStairMax(settingDTO.getStairMax().intValue());
                }
                salaryGroupProjectSettings.add(setting);
            }
        }
        salaryGroupProjectSettinService.saveBatch(salaryGroupProjectSettings);
    }

    /**
     * 删除薪酬组配置（根本没有被使用的薪酬组配置会被逻辑删除，如为次月添加3次配置，之前两次根本没有被用，直接删除）
     *
     * @param salaryGroups
     */
    @Transactional(rollbackFor = Exception.class)
    public void delSalaryGroups(List<SalaryGroup> salaryGroups) {
        if (PublicUtil.isEmpty(salaryGroups)) {
            return;
        }
        List<Long> salaryGroupIds =  salaryGroups.stream().map(SalaryGroup::getId).collect(Collectors.toList());
//        salaryGroupService.removeByIds(kpiIds);
        salaryGroupService.update(Wrappers.<SalaryGroup>lambdaUpdate()
                .set(SalaryGroup::getYn, Boolean.FALSE)
                .in(SalaryGroup::getId, salaryGroupIds)
        );
    }

    /**
     * 修改多个薪酬组状态
     *
     * @param salaryGroups
     * @param status
     */
    @Transactional(rollbackFor = Exception.class)
    public void modifyStatusBySalaryGroups(List<SalaryGroup> salaryGroups, SettingStatusEnum status) {
        if (PublicUtil.isEmpty(salaryGroups)) {
            return;
        }
        List<Long> kpiIds =  salaryGroups.stream().map(SalaryGroup::getId).collect(Collectors.toList());
        this.modifyStatusByIds(kpiIds, status);
    }

    /**
     * 修改多个薪酬组状态
     *
     * @param salaryGroupIds
     * @param status
     */
    @Transactional(rollbackFor = Exception.class)
    public void modifyStatusByIds(List<Long> salaryGroupIds, SettingStatusEnum status) {
        if (PublicUtil.isEmpty(salaryGroupIds)) {
            return;
        }
        log.info("修改薪酬组配置状态：{}，{}", JSON.toJSONString(salaryGroupIds), status.getName());
        LocalDate localDate = LocalDate.now();
        LambdaUpdateWrapper<SalaryGroup> updateWrapper = Wrappers.<SalaryGroup>lambdaUpdate()
                .set(SalaryGroup::getStatus, status)
                .in(SalaryGroup::getId, salaryGroupIds);
        switch (status) {
            case EFFECTIVE : {
                updateWrapper.set(SalaryGroup::getBeginTime, localDate);
                break;
            }
            case INEFFECTIVE : {
                updateWrapper.set(SalaryGroup::getOverTime, PublicUtil.getPreviousDay(localDate));
                break;
            }
        }
        salaryGroupService.update(updateWrapper);
    }

    /**
     * 修改薪酬组状态
     *
     * @param salaryGroup
     * @param status
     */
    @Transactional(rollbackFor = Exception.class)
    public void modifyStatusById(SalaryGroup salaryGroup, SettingStatusEnum status) {
        log.info("修改薪酬组配置状态：{}，{}", salaryGroup.getId(), status.getName());
        LocalDate localDate = LocalDate.now();
        LambdaUpdateWrapper<SalaryGroup> updateWrapper = Wrappers.<SalaryGroup>lambdaUpdate()
                .set(SalaryGroup::getStatus, status)
                .eq(SalaryGroup::getId, salaryGroup.getId());
        switch (status) {
//            case BE_EFFECTIVE : {
//                SalaryGroup SalaryGroup = kpiGroupService.getById(kpiId);
//                updateWrapper.set(SalaryGroup::getBeginTime, SalaryGroup.getBeginTime());
//                break;
//            }
            case EFFECTIVE : {
                updateWrapper.set(SalaryGroup::getBeginTime, localDate);
                break;
            }
            case INEFFECTIVE : {
                //之前生效过得配置需要 设置结束时间
                if (SettingStatusEnum.EFFECTIVE.equals(salaryGroup.getStatus())) {
                    updateWrapper.set(SalaryGroup::getOverTime, PublicUtil.getPreviousDay(localDate));
                }
                break;
            }
        }
        salaryGroupService.update(updateWrapper);
    }

    /**
     * 初始化sgc
     */
    @Transactional(rollbackFor = Exception.class)
    public void initSgc() {
        List<SettingStatusEnum> status = new ArrayList<SettingStatusEnum>(){{
            add(SettingStatusEnum.EFFECTIVE);
        }};
        List<SalaryGroup> salaryGroups = salaryGroupService.list(Wrappers.<SalaryGroup>lambdaQuery()
                .in(SalaryGroup::getStatus, status)
                .eq(SalaryGroup::getYn, Boolean.TRUE)
        );

        for (SalaryGroup salaryGroup : salaryGroups) {
            final Long salaryGroupId = salaryGroup.getId();
            final String sgc = PublicUtil.getUUID();

            salaryGroupService.update(Wrappers.<SalaryGroup>lambdaUpdate()
                    .set(SalaryGroup::getSgc, sgc)
                    .set(SalaryGroup::getUpdateTime, new Date())
                    .eq(SalaryGroup::getId, salaryGroupId)
            );
            salaryGroupUserService.update(Wrappers.<SalaryGroupUser>lambdaUpdate()
                    .set(SalaryGroupUser::getSgc, sgc)
                    .set(SalaryGroupUser::getUpdateTime, new Date())
                    .eq(SalaryGroupUser::getSalaryGroupId, salaryGroupId)
            );
            salaryPoolService.update(Wrappers.<SalaryPool>lambdaUpdate()
                    .set(SalaryPool::getSgc, sgc)
                    .set(SalaryPool::getUpdateTime, new Date())
                    .eq(SalaryPool::getSalaryGroupId, salaryGroupId)
            );
        }
    }

}
