package cn.fw.morax.service.biz.salary;

import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.PayrollRecord;
import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.dto.SalaryExtraDTO;
import cn.fw.morax.domain.dto.SalaryRawDataDTO;
import cn.fw.morax.domain.enums.ExtraSalaryTypeEnum;
import cn.fw.morax.domain.enums.SalaryPayrollStatusEnum;
import cn.fw.morax.domain.enums.SalaryValueTypeEnum;
import cn.fw.morax.rpc.oop.OopRpcService;
import cn.fw.morax.rpc.oop.dto.ShopDTO;
import cn.fw.morax.sdk.dto.CustomList;
import cn.fw.morax.sdk.dto.salary.*;
import cn.fw.morax.service.data.PayrollRecordService;
import cn.fw.morax.service.data.salary.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryReportService
 * @description : 薪酬数据上报
 * @date : 2022-04-22 10:28
 */

@RequiredArgsConstructor
@Slf4j
@Service
public class SalaryReportService {

    private final SalaryRawDataService salaryRawDataService;
    private final SalaryExtraProjectService salaryExtraProjectService;
    private final SalaryGroupUserService salaryGroupUserService;
    private final SalaryPoolService salaryPoolService;
    private final OopRpcService oopRpcService;
    private final PayrollRecordService payrollRecordService;
    private final SalaryProjectService salaryProjectService;
    private final SalaryGroupService salaryGroupService;
    private final SalaryPoolCommonService salaryPoolCommonService;

    private final StringRedisTemplate stringRedisTemplate;
    @Value("${spring.cache.custom.global-prefix}:extra-salary")
    @Getter
    private String extraSalaryKey;

    @Value("${spring.cache.custom.global-prefix}:push-money-salary")
    @Getter
    private String pushMoneySalaryKey;


    /**
     * 提成数据保存失败的重试
     */
    public void retrySavePushMoney() {
        ListOperations<String, String> listOps = stringRedisTemplate.opsForList();
        String str;
        List<String> failList = new ArrayList<>();
        while ((str = listOps.leftPop(getPushMoneySalaryKey())) != null) {
            try {
                SalaryRawDataDTO dto = JSONObject.parseObject(str, SalaryRawDataDTO.class);
                if (Objects.isNull(dto)) {
                    continue;
                }
                savePushMoneyData(dto);
            } catch (Exception e) {
                failList.add(str);
            }
        }

        if (!CollectionUtils.isEmpty(failList)) {
            listOps.rightPushAll(getPushMoneySalaryKey(), failList);
        }
    }


    /**
     * 额外薪资数据保存失败的重试
     */
    public void retrySaveReward() {
        ListOperations<String, String> listOps = stringRedisTemplate.opsForList();
        String str;
        List<String> failList = new ArrayList<>();
        while ((str = listOps.leftPop(getExtraSalaryKey())) != null) {
            try {
                SalaryExtraDTO dto = JSONObject.parseObject(str, SalaryExtraDTO.class);
                if (Objects.isNull(dto)) {
                    continue;
                }
                saveExtraProject(dto);
            } catch (Exception e) {
                failList.add(str);
            }
        }

        if (!CollectionUtils.isEmpty(failList)) {
            listOps.rightPushAll(getPushMoneySalaryKey(), failList);
        }
    }

    /**
     * 提成类型薪酬项上报
     *
     * @param salaryValueList
     */
    public void pushMoneySalaryReport(CustomList<SalaryDataReportReq> salaryValueList) {
        for (SalaryDataReportReq dataReportReq : salaryValueList) {
            SalaryRawDataDTO royalties = transferRoyalties(dataReportReq);
            CompletableFuture.runAsync(() -> pushMoneySalaryReport(royalties));
        }
    }


    /**
     * 持久化提成数据
     *
     * @param royalties
     */
    private void pushMoneySalaryReport(SalaryRawDataDTO royalties) {
        try {
            savePushMoneyData(royalties);
        } catch (Exception e) {
            cacheFailProject(royalties, getPushMoneySalaryKey());
            log.error("提成类型薪酬项上报数据处理失败", e);
        }
    }


    /**
     * 奖惩项薪资上报
     *
     * @param req
     */
    public void salaryRewardReport(SalaryRewardReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req);
        try {
            saveExtraProject(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("奖惩项薪资上报数据处理失败", e);
        }
    }

    /**
     * 社保项上报
     *
     * @param req
     */
    public void socialSecurityProject(GeneralSalaryProjectReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req, Boolean.FALSE, ExtraSalaryTypeEnum.NECESSARY);
        try {
            saveExtraProject(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("社保项上报数据处理失败", e);
        }
    }

    /**
     * 个税上报
     *
     * @param req
     */
    public void personTaxProject(GeneralSalaryProjectReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req, Boolean.FALSE, ExtraSalaryTypeEnum.PERSON_TAX);
        try {
            saveExtraProject(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("社保项上报数据处理失败", e);
        }
    }

    /**
     * 补贴项上报
     *
     * @param req
     */
    public void subsidyProject(GeneralSalaryProjectReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req, Boolean.TRUE, ExtraSalaryTypeEnum.SUBSIDY);
        try {
            saveExtraProject(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("补贴项薪资上报数据处理失败", e);
        }
    }

    /**
     * 保存提成项数据
     *
     * @param royalties
     */
    @Transactional(rollbackFor = Exception.class)
    public void savePushMoneyData(SalaryRawDataDTO royalties) {
        SalaryRawData rawData = new SalaryRawData();
        rawData.setSalaryProjectCode(royalties.getSalaryProjectCode());
        rawData.setSalaryProjectValue(royalties.getSalaryProjectValue());
        rawData.setValueType(SalaryValueTypeEnum.ofValue(royalties.getValueType()));
        rawData.setDataDate(DateUtil.date2LocalDate(royalties.getDataDate()));
        rawData.setUserId(royalties.getUserId());
        rawData.setGroupId(royalties.getGroupId());
        rawData.setYn(Boolean.TRUE);
        salaryRawDataService.update(Wrappers.<SalaryRawData>lambdaUpdate()
                .set(SalaryRawData::getYn, Boolean.FALSE)
                .eq(SalaryRawData::getUserId, royalties.getUserId())
                .eq(SalaryRawData::getDataDate, DateUtil.date2LocalDate(royalties.getDataDate()))
                .eq(SalaryRawData::getSalaryProjectCode, royalties.getSalaryProjectCode())
                .eq(SalaryRawData::getGroupId, royalties.getGroupId())
                .eq(SalaryRawData::getValueType, SalaryValueTypeEnum.ofValue(royalties.getValueType()))
        );
        salaryRawDataService.save(rawData);
    }


    /**
     * 保存额外薪酬项
     *
     * @param extraDTO
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveExtraProject(SalaryExtraDTO extraDTO) {
        SalaryPool salaryPool = queryPoolByUser(extraDTO.getUserId(), extraDTO.getPostId(), extraDTO.getShopId(), extraDTO.getDataDate());
        Long fundShopId = PublicUtil.isNotEmpty(salaryPool.getFundShopId()) ?
                salaryPool.getFundShopId() : salaryPool.getShopId();
        SalaryExtraProject extraProject = transferBaseExtra(extraDTO);
        if (Objects.isNull(extraProject)) {
            return;
        }
        extraProject.setPostId(extraDTO.getPostId());
        extraProject.setShopId(fundShopId);
        extraProject.setGroupId(salaryPool.getGroupId());
        salaryExtraProjectService.save(extraProject);
        if (Boolean.TRUE.equals(extraDTO.getTimely())) {
            Long dealerId = Optional.ofNullable(oopRpcService.queryShop(fundShopId)).map(ShopDTO::getDealerId).orElse(null);
            PayrollRecord pr = PayrollRecord.builder()
                    .userId(extraProject.getUserId())
                    .userName(salaryPool.getUserName())
                    .shopId(salaryPool.getShopId())
                    .fundShopId(fundShopId)
                    .dealerId(dealerId)
                    .bizId(String.valueOf(extraProject.getId()))
                    .payment(ExtraSalaryTypeEnum.AWARD.equals(extraProject.getSalaryType()) || ExtraSalaryTypeEnum.SUBSIDY.equals(extraProject.getSalaryType()))
                    .amount(extraProject.getMoney())
                    .salary(Boolean.FALSE)
                    .payrollStatus(SalaryPayrollStatusEnum.WAITING)
                    .monthly(YearMonth.from(extraProject.getDataDate()))
                    .groupId(salaryPool.getGroupId())
                    .build();
            payrollRecordService.save(pr);
        }
    }

    /**
     * 查询薪酬池
     *
     * @param userId
     * @param postId
     * @param shopId
     * @param date
     * @return
     */
    private SalaryPool queryPoolByUser(Long userId, Long postId, Long shopId, Date date) {
        SalaryGroupUser user = salaryGroupUserService.queryUserByPostShopAndUserId(userId, postId, shopId, date);
        BV.notNull(user, () -> "用户薪酬组不能为空");
        SalaryPool salaryPool = salaryPoolCommonService.inspectionPool(user);
        BV.notNull(salaryPool, () -> "用户薪酬池不能为空");
        return salaryPool;
    }

    /**
     * 缓存处理失败的数据
     *
     * @param dto
     */
    private <T> void cacheFailProject(T dto, String key) {
        BoundListOperations<String, String> listOps = stringRedisTemplate.boundListOps(key);
        String jsonString = JSONObject.toJSONString(dto);
        listOps.rightPush(jsonString);
    }

    /**
     * 对象转换
     *
     * @param dto SalaryExtraDTO
     * @return SalaryExtraProject
     */
    private SalaryExtraProject transferBaseExtra(SalaryExtraDTO dto) {
        if (Objects.isNull(dto)) {
            return null;
        }
        SalaryExtraProject salaryExtraProject = new SalaryExtraProject();
        salaryExtraProject.setUserId(dto.getUserId());
        salaryExtraProject.setMoney(dto.getMoney());
        salaryExtraProject.setSalaryType(dto.getSalaryType());
        salaryExtraProject.setDataId(dto.getDataId());
        salaryExtraProject.setDataName(dto.getDataName());
        salaryExtraProject.setDataDate(DateUtil.date2LocalDate(dto.getDataDate()));
        salaryExtraProject.setTimely(dto.getTimely());
        return salaryExtraProject;
    }


    /**
     * 对象转换
     *
     * @param req SalaryRewardReq
     * @return SalaryExtraDTO
     */
    private SalaryExtraDTO transferExtraProject(SalaryRewardReq req) {
        SalaryExtraDTO dto = new SalaryExtraDTO();
        dto.setUserId(req.getUserId());
        dto.setMoney(req.getAmount());
        dto.setSalaryType(ExtraSalaryTypeEnum.AWARD);
        if (!Boolean.TRUE.equals(req.getAward())) {
            dto.setMoney(req.getAmount().multiply(BigDecimal.valueOf(-1)));
            dto.setSalaryType(ExtraSalaryTypeEnum.PENALTY);
        }
        dto.setDataId(req.getDataId());
        dto.setDataName(req.getDataName());
        dto.setDataDate(req.getDataTime());
        dto.setPostId(req.getPostId());
        dto.setShopId(req.getShopId());
        dto.setTimely(req.getTimely());
        return dto;
    }

    /**
     * 对象转换
     *
     * @param req      SalaryRewardReq
     * @param addition
     * @return SalaryExtraDTO
     */
    private SalaryExtraDTO transferExtraProject(GeneralSalaryProjectReq req, boolean addition, ExtraSalaryTypeEnum salaryTypeEnum) {
        SalaryExtraDTO dto = new SalaryExtraDTO();
        dto.setUserId(req.getUserId());
        dto.setSalaryType(salaryTypeEnum);
        if (addition) {
            dto.setMoney(req.getAmount());
        } else {
            dto.setMoney(req.getAmount().multiply(BigDecimal.valueOf(-1)));
        }
        dto.setDataId(req.getDataId());
        dto.setDataName(req.getDataName());
        dto.setDataDate(req.getDataTime());
        dto.setPostId(req.getPostId());
        dto.setShopId(req.getShopId());
        dto.setTimely(Boolean.FALSE);
        return dto;
    }

    /**
     * 转换提成数据
     *
     * @param req
     * @return
     */
    private SalaryRawDataDTO transferRoyalties(SalaryDataReportReq req) {
        SalaryRawDataDTO dto = new SalaryRawDataDTO();
        dto.setSalaryProjectCode(req.getSalaryProjectCode());
        dto.setUserId(req.getUserId());
        dto.setDataDate(req.getDataDate());
        dto.setGroupId(req.getGroupId());
        dto.setValueType(req.getValueType());

        List<SalaryValueDTO> value = req.getSalaryValue();
        Map<String, BigDecimal> map = new HashMap<>(value.size());

        for (SalaryValueDTO valueDTO : value) {
            map.put(valueDTO.getKey(), valueDTO.getValue());
        }
        dto.setSalaryProjectValue(JSONObject.toJSONString(map));
        return dto;
    }

    /**
     * 查询薪酬组人员
     *
     * @param salaryGroupUserQuery
     * @return
     */
    public List<SalaryGroupUserResult> querySalaryGroupUser(SalaryGroupUserQuery salaryGroupUserQuery) {
        final String salaryCode = salaryGroupUserQuery.getSalaryCode();
        final Long postId = salaryGroupUserQuery.getPostId();
        final Date dataDate = new Date(salaryGroupUserQuery.getDataDate());
        log.info("查询薪酬组人员，日期：{}", new SimpleDateFormat("yyyy-MM-dd").format(dataDate));

        SalaryProject salaryProject = salaryProjectService.queryByCode(salaryCode, Boolean.FALSE);
        BV.notNull(salaryProject, () -> "指标码不存在或已失效");

        ArrayList<SalaryGroupUserResult> list = new ArrayList<>();

        List<Long> salaryGroupIds = salaryGroupService.querySalaryGroupIdsByCode(salaryCode,
                postId, salaryGroupUserQuery.getGroupId(), dataDate);

        if (CollectionUtils.isEmpty(salaryGroupIds)) {
            log.error("没有对应的薪酬组数据【{}】", salaryGroupIds);
            return list;
        }

        List<SalaryGroupUser> groupUsers = salaryGroupUserService.list(Wrappers.<SalaryGroupUser>lambdaQuery()
                .in(SalaryGroupUser::getSalaryGroupId, new HashSet<>(salaryGroupIds))
                .eq(SalaryGroupUser::getDataDate, DateUtil.date2LocalDate(dataDate))
        );

        if (CollectionUtils.isEmpty(groupUsers)) {
            log.error("没有对应日期的薪酬组人员数据【薪酬组id：{}，日期：{}】", JSON.toJSONString(salaryGroupIds), DateUtil.getStringDateShort(dataDate));
            return list;
        }

        Map<Long, List<SalaryGroupUser>> salaryGroupUserMap = groupUsers.stream().collect(Collectors.groupingBy(SalaryGroupUser::getSalaryGroupId));
        for (List<SalaryGroupUser> valueArr : salaryGroupUserMap.values()) {
            SalaryGroupUserResult result = new SalaryGroupUserResult();
            result.setRoleCodeList(salaryProject.getRoleCodes());
            result.setUserIdList(transferUser(valueArr));
            list.add(result);
        }

        return list;
    }

    /**
     * 转换器  SalaryGroupUser -> SimpleUserData
     *
     * @param groupUsers
     * @return
     */
    private List<SalaryGroupUserResult.SimpleUserData> transferUser(List<SalaryGroupUser> groupUsers) {
        List<SalaryGroupUserResult.SimpleUserData> list = new ArrayList<>();
        for (SalaryGroupUser salaryGroupUser : groupUsers) {
            SalaryGroupUserResult.SimpleUserData simpleUserData = new SalaryGroupUserResult.SimpleUserData();
            simpleUserData.setUserId(salaryGroupUser.getUserId());
            simpleUserData.setUserName(salaryGroupUser.getUserName());
            simpleUserData.setShopId(salaryGroupUser.getShopId());
            simpleUserData.setShopName(salaryGroupUser.getShopName());
            list.add(simpleUserData);
        }

        return list;
    }

    public SalaryGroupUserResult queryGroupSalaryUser(SalaryGroupUserQuery salaryGroupUserQuery) {
        final Long groupId = salaryGroupUserQuery.getGroupId();
        final Date dataDate = new Date(salaryGroupUserQuery.getDataDate());
        log.info("查询薪酬组人员，日期：{}", new SimpleDateFormat("yyyy-MM-dd").format(dataDate));
        final YearMonth monthly = YearMonth.from(DateUtil.date2LocalDateTime(dataDate));
        List<SalaryGroupUserResult.SimpleUserData> userData = new ArrayList<>();

        List<SalaryPool> pools = salaryPoolService.list(Wrappers.<SalaryPool>lambdaQuery()
                .eq(SalaryPool::getGroupId, groupId)
                .eq(SalaryPool::getMonthly, monthly)
                .eq(SalaryPool::getYn, Boolean.TRUE)
        );
        SalaryGroupUserResult result = new SalaryGroupUserResult();
        if (PublicUtil.isEmpty(pools)) {
            result.setUserIdList(new ArrayList<>());
            return result;
        }
        for (SalaryPool pool : pools) {
            SalaryGroupUserResult.SimpleUserData simpleUserData = new SalaryGroupUserResult.SimpleUserData();
            simpleUserData.setUserId(pool.getUserId());
            simpleUserData.setUserName(pool.getUserName());
            simpleUserData.setShopId(pool.getShopId());
            simpleUserData.setShopName(pool.getShopName());
            userData.add(simpleUserData);
        }
        result.setUserIdList(userData);
        return result;
    }
}
