package cn.fw.morax.service.component;

import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.pojo.event.PaySalaryEvent;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.SalaryGeneralSettin;
import cn.fw.morax.domain.dto.SalaryPayNoticeDTO;
import cn.fw.morax.domain.enums.SalaryConfirmStatusEnum;
import cn.fw.morax.domain.vo.salary.SalaryConfirmVO;
import cn.fw.morax.rpc.backlog.TodoRpcService;
import cn.fw.morax.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.morax.rpc.erp.ErpRpcService;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleInfoDTO;
import cn.fw.morax.service.biz.CommonService;
import cn.fw.morax.service.data.salary.SalaryConfirmService;
import cn.fw.morax.service.data.salary.SalaryGeneralSettinService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.eventbus.Subscribe;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.YearMonth;
import java.util.*;

@Component
@Slf4j
@RequiredArgsConstructor
public class PaySalaryObserver {

    private final SalaryGeneralSettinService salaryGeneralSettinService;
    private final SalaryConfirmService salaryConfirmService;
    private final ErpRpcService erpRpcService;
    private final TodoRpcService todoRpcService;
    private final CommonService commonService;

    @Value("${todo.pay-salary-notice}")
    @Getter
    private String paySalaryTodoCode;


    /**
     * 工资确认后通知支付管理人员
     *
     * @param event
     */
    @Subscribe
    public void sendPaySalaryNotice(PaySalaryEvent event) {
        log.info("发送工资结算待办[{}]", event);
        Long shopId = event.getShopId();
        YearMonth yearMonth = event.getMonthly();
        List<Long> staffIds = commonService.queryShopLeaveStaffIds(shopId, yearMonth);
        //离职员工不会确认，需要排除上月已经离职员工
        Integer noConfirmCount = salaryConfirmService.getCountShopNoConfirm(shopId, SalaryConfirmStatusEnum.CONFIRMED, staffIds);
        if (noConfirmCount > 1) {
            return;
        }
        List<SalaryConfirmVO> allConfirms = salaryConfirmService.getShopSalaryConfirm(shopId, SalaryConfirmStatusEnum.CONFIRMED);
        BigDecimal totalReward = allConfirms.stream().map(SalaryConfirmVO::getReward).reduce(BigDecimal.ZERO,BigDecimal::add);
        Integer staffNum = allConfirms.size();
        SalaryPayNoticeDTO dto = SalaryPayNoticeDTO.builder()
                .shopId(shopId)
                .staffNum(staffNum)
                .shopName(event.getShopName())
                .totalReward(totalReward)
                .monthly(event.getMonthly())
                .groupId(event.getGroupId())
                .id(PublicUtil.getUUID())
                .build();
        this.sendPaySalaryTodo(dto);
    }

    /**
     * 发送工资结算代办通知
     *
     * @param dto
     */
    private void sendPaySalaryTodo(SalaryPayNoticeDTO dto) {
        Long shopId = dto.getShopId();
        List<RpcUserRoleInfoDTO> userRoleInfoDTOS = erpRpcService.getUserRoleInfo(Constant.ROLE_CODE_WAGE_PAYMENT, shopId);
        if (PublicUtil.isEmpty(userRoleInfoDTOS)) {
            log.info("门店没有工资发放角色员工，shopId:{}", shopId);
            return;
        }
        SalaryGeneralSettin setting = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaUpdate()
                .eq(SalaryGeneralSettin::getGroupId, dto.getGroupId())
        );
        if (PublicUtil.isEmpty(setting)) {
            log.error("集团薪酬通用配置未设置，不能推送发放工资代办， groupId:{}", dto.getGroupId());
            return;
        }

        LocalDate payOffDate = LocalDate.now().withDayOfMonth(setting.getPayoffDate());
        LocalDateTime expireTime = LocalDateTime.of(payOffDate, LocalTime.MIN);
        RpcUserRoleInfoDTO user = userRoleInfoDTOS.get(0);
        try {
            Map<String, String> dynamicMap = new HashMap<>(5);
            dynamicMap.put("shopName", dto.getShopName());
            dynamicMap.put("monthly", dto.getMonthly().toString());
            dynamicMap.put("totalReward", dto.getTotalReward().toString());
            dynamicMap.put("staffNum", dto.getStaffNum().toString());
            BackLogItemDTO backLogItemDTO = new BackLogItemDTO(user.getUserId(), getPaySalaryTodoCode(), PublicUtil.getUUID(), new Date(), shopId);
            backLogItemDTO.setExpireTime(DateUtil.localDateTime2Date(expireTime));
            backLogItemDTO.setDynamicMap(dynamicMap);
            boolean pushed = todoRpcService.push(backLogItemDTO);
        } catch (Exception e) {
            log.error("推送工资支付待办失败，shopId:{}", shopId, e);
        }
    }
}